@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2003 CollabNet.  All rights reserved.
+ * Copyright (c) 2003-2004 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -19,7 +19,19 @@ package org.tigris.subversion.javahl;
 
 import java.util.Date;
 
+/**
+ * this interface is used to receive every single line for a file on a
+ * the SVNClientinterface.blame call.
+ */
 public interface BlameCallback
 {
-    public void singleLine(Date changed, long revision, String author, String line);
+    /**
+     * the method will be called for every line in a file.
+     * @param changed   the date of the last change.
+     * @param revision  the revision of the last change.
+     * @param author    the author of the last change.
+     * @param line      the line in the file
+     */
+    public void singleLine(Date changed, long revision, String author,
+                           String line);
 }
