@@ -22,13 +22,9 @@
 
 /*** Includes. ***/
 
-#include "svn_private_config.h"
 #include "svn_cmdline.h"
-#include "svn_wc.h"
 #include "svn_pools.h"
 #include "svn_client.h"
-#include "svn_string.h"
-#include "svn_delta.h"
 #include "svn_error.h"
 #include "svn_utf.h"
 #include "svn_path.h"
@@ -57,11 +53,8 @@ svn_cl__propdel (apr_getopt_t *os,
   SVN_ERR (svn_utf_cstring_to_utf8 (&pname_utf8, pname, pool));
 
   /* Suck up all the remaining arguments into a targets array */
-  SVN_ERR (svn_opt_args_to_target_array (&targets, os,
-                                         opt_state->targets,
-                                         &(opt_state->start_revision),
-                                         &(opt_state->end_revision),
-                                         FALSE, pool));
+  SVN_ERR (svn_opt_args_to_target_array2 (&targets, os,
+                                          opt_state->targets, pool));
 
   /* Add "." if user passed 0 file arguments */
   svn_opt_push_implicit_dot_target (targets, pool);
@@ -69,26 +62,10 @@ svn_cl__propdel (apr_getopt_t *os,
   if (opt_state->revprop)  /* operate on a revprop */
     {
       svn_revnum_t rev;
-      const char *URL, *target;
+      const char *URL;
 
-      /* All property commands insist on a specific revision when
-         operating on a revprop. */
-      if (opt_state->start_revision.kind == svn_opt_revision_unspecified)
-        return svn_cl__revprop_no_rev_error (pool);
-
-      /* Else some revision was specified, so proceed. */
-
-      /* Either we have a URL target, or an implicit wc-path ('.')
-         which needs to be converted to a URL. */
-      if (targets->nelts <= 0)
-        return svn_error_create(SVN_ERR_CL_INSUFFICIENT_ARGS, NULL,
-                                _("No URL target available"));
-      target = ((const char **) (targets->elts))[0];
-      SVN_ERR (svn_client_url_from_path (&URL, target, pool));  
-      if (URL == NULL)
-        return svn_error_create
-          (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-           _("Either a URL or versioned item is required"));
+      SVN_ERR (svn_cl__revprop_prepare (&opt_state->start_revision, targets,
+                                        &URL, pool));
 
       /* Let libsvn_client do the real work. */
       SVN_ERR (svn_client_revprop_set (pname_utf8, NULL,
@@ -117,12 +94,22 @@ svn_cl__propdel (apr_getopt_t *os,
       for (i = 0; i < targets->nelts; i++)
         {
           const char *target = ((const char **) (targets->elts))[i];
+          svn_boolean_t success;
 
           svn_pool_clear (subpool);
           SVN_ERR (svn_cl__check_cancel (ctx->cancel_baton));
-          SVN_ERR (svn_client_propset (pname_utf8, NULL, target,
-                                       opt_state->recursive, subpool));
-          if (! opt_state->quiet) 
+
+          /* Pass FALSE for 'skip_checks' because it doesn't matter here,
+             and opt_state->force doesn't apply to this command anyway. */
+          SVN_ERR (svn_cl__try (svn_client_propset2 (pname_utf8, NULL, target,
+                                                     opt_state->recursive,
+                                                     FALSE, ctx, subpool),
+                                &success, opt_state->quiet,
+                                SVN_ERR_UNVERSIONED_RESOURCE,
+                                SVN_ERR_ENTRY_NOT_FOUND,
+                                SVN_NO_ERROR));
+          
+          if (success && (! opt_state->quiet))
             {
               SVN_ERR (svn_cmdline_printf
                        (subpool, opt_state->recursive
