@@ -39,7 +39,7 @@ extern "C" {
 
 /** Subversion error object.
  *
- * Defined here, rather than in svn_error.h, to avoid a recursive #include 
+ * Defined here, rather than in svn_error.h, to avoid a recursive @#include 
  * situation.
  */
 typedef struct svn_error_t
@@ -66,12 +66,23 @@ typedef struct svn_error_t
 
 
 
+/** @defgroup APR_ARRAY_compat_macros APR Array Compatibility Helper Macros
+ * These macros are provided by APR itself from version 1.3.
+ * Definitions are provided here for when using older versions of APR.
+ * @{
+ */
+
 /** index into an apr_array_header_t */
+#ifndef APR_ARRAY_IDX
 #define APR_ARRAY_IDX(ary,i,type) (((type *)(ary)->elts)[i])
+#endif
 
 /** easier array-pushing syntax */
+#ifndef APR_ARRAY_PUSH
 #define APR_ARRAY_PUSH(ary,type) (*((type *)apr_array_push (ary)))
+#endif
 
+/** @} */
 
 /** The various types of nodes in the Subversion filesystem. */
 typedef enum
@@ -87,9 +98,44 @@ typedef enum
 
   /* something's here, but we don't know what */
   svn_node_unknown
-
 } svn_node_kind_t;
 
+/** About Special Files in Subversion
+ *
+ * Subversion denotes files that cannot be portably created or
+ * modified as "special" files (svn_node_special).  It stores these
+ * files in the repository as a plain text file with the svn:special
+ * property set.  The file contents contain: a platform-specific type
+ * string, a space character, then any information necessary to create
+ * the file on a supported platform.  For example, if a symbolic link
+ * were being represented, the repository file would have the
+ * following contents:
+ *
+ * "link /path/to/link/target"
+ *
+ * Where 'link' is the identifier string showing that this special
+ * file should be a symbolic link and '/path/to/link/target' is the
+ * destination of the symbolic link.
+ *
+ * Special files are stored in the text-base exactly as they are
+ * stored in the repository.  The platform specific files are created
+ * in the working copy at EOL/keyword translation time using
+ * svn_subst_copy_and_translate2().  If the current platform does not
+ * support a specific special file type, the file is copied into the
+ * working copy as it is seen in the repository.  Because of this,
+ * users of other platforms can still view and modify the special
+ * files, even if they do not have their unique properties.
+ *
+ * New types of special files can be added by:
+ *  1. Implementing a platform-dependent routine to create a uniquely
+ *     named special file and one to read the special file in
+ *     libsvn_subr/io.c.
+ *  2. Creating a new textual name similar to
+ *     SVN_SUBST__SPECIAL_LINK_STR in libsvn_subr/subst.c.
+ *  3. Handling the translation/detranslation case for the new type in
+ *     create_special_file and detranslate_special_file, using the
+ *     routines from 1.
+ */
 
 /** A revision number. */
 typedef long int svn_revnum_t;
@@ -109,7 +155,9 @@ typedef long int svn_revnum_t;
 /** Convert null-terminated C string @a str to a revision number. */
 #define SVN_STR_TO_REV(str) ((svn_revnum_t) atol(str))
 
-/** In @c printf()-style functions, format revision numbers using this. */
+/** In printf()-style functions, format revision numbers using this.
+ * Do not use this macro within the Subversion project source code, because
+ * the language translation tools have trouble parsing it. */
 #define SVN_REVNUM_T_FMT "ld"
 
 
@@ -119,13 +167,15 @@ typedef apr_int64_t svn_filesize_t;
 /** The 'official' invalid file size constant. */
 #define SVN_INVALID_FILESIZE ((svn_filesize_t) -1)
 
-/** In @c printf()-style functions, format file sizes using this. */
+/** In printf()-style functions, format file sizes using this. */
 #define SVN_FILESIZE_T_FMT APR_INT64_T_FMT
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 /* Parse a base-10 numeric string into a 64-bit unsigned numeric value. */
 /* NOTE: Private. For use by Subversion's own code only. See issue #1644. */
 /* FIXME: APR should supply a function to do this, such as "apr_atoui64". */
 #define svn__atoui64(X) ((apr_uint64_t) apr_atoi64(X))
+#endif
 
 
 /** YABT:  Yet Another Boolean Type */
@@ -219,6 +269,10 @@ typedef struct svn_dirent_t
 /** Short version of LastChangedRevision */
 #define SVN_KEYWORD_REVISION_SHORT   "Rev"
 
+/** Medium version of LastChangedRevision, matching the one CVS uses.
+ * @since New in 1.1. */
+#define SVN_KEYWORD_REVISION_MEDIUM  "Revision"
+
 /** The most recent date (repository time) when this file was changed. */
 #define SVN_KEYWORD_DATE_LONG        "LastChangedDate"
 
@@ -237,15 +291,53 @@ typedef struct svn_dirent_t
 /** Short version of HeadURL */
 #define SVN_KEYWORD_URL_SHORT        "URL"
 
-/** A compressed combination of the other four keywords.
- *
- * (But see comments above about a more general solution to keyword
- * combinations.)
- */
+/** A compressed combination of the other four keywords. */
 #define SVN_KEYWORD_ID               "Id"
 
 /** @} */
 
+
+/** All information about a commit.
+ *
+ * @note Objects of this type should always be created using the
+ * svn_create_commit_info() function.
+ *
+ * @since New in 1.3.
+ */
+typedef struct svn_commit_info_t
+{
+  /** just-committed revision. */
+  svn_revnum_t revision;
+
+  /** server-side date of the commit. */
+  const char *date;
+
+  /** author of the commit. */
+  const char *author;
+
+  /** error message from post-commit hook, or NULL. */
+  const char *post_commit_err;
+
+} svn_commit_info_t;
+
+
+/**
+ * Allocate an object of type @c svn_commit_info_t in @a pool and
+ * return it.
+ *
+ * The @c revision field of the new struct is set to @c
+ * SVN_INVALID_REVNUM.  All other fields are initialized to @c NULL.
+ *
+ * @note Any object of the type @c svn_commit_info_t should
+ * be created using this function.
+ * This is to provide for extending the svn_commit_info_t in
+ * the future.
+ *
+ * @since New in 1.3.
+ */
+svn_commit_info_t *
+svn_create_commit_info (apr_pool_t *pool);
+
 
 /** A structure to represent a path that changed for a log entry. */
 typedef struct svn_log_changed_path_t
@@ -262,8 +354,17 @@ typedef struct svn_log_changed_path_t
 } svn_log_changed_path_t;
 
 
+/**
+ * Return a deep copy of @a changed_path, allocated in @a pool.
+ *
+ * @since New in 1.3.
+ */
+svn_log_changed_path_t *svn_log_changed_path_dup (
+  const svn_log_changed_path_t *changed_path, apr_pool_t *pool);
+
+
 /** The callback invoked by log message loopers, such as
- * @c svn_ra_plugin_t.get_log() and @c svn_repos_get_logs().
+ * @c svn_ra_plugin_t.get_log() and svn_repos_get_logs().
  *
  * This function is invoked once on each log message, in the order
  * determined by the caller (see above-mentioned functions).
@@ -272,18 +373,18 @@ typedef struct svn_log_changed_path_t
  * think they are.  Any of @a author, @a date, or @a message may be @c NULL.
  *
  * If @a date is neither null nor the empty string, it was generated by
- * @c svn_time_to_string() and can be converted to @c apr_time_t with
- * @c svn_time_from_string().
+ * svn_time_to_string() and can be converted to @c apr_time_t with
+ * svn_time_from_string().
  *
  * If @a changed_paths is non-@c NULL, then it contains as keys every path
  * committed in @a revision; the values are (@c svn_log_changed_path_t *) 
- * structures (see above).
+ * structures.
  *
  * ### The only reason @a changed_paths is not qualified with `const' is
- * that we usually want to loop over it, and @c apr_hash_first() doesn't
+ * that we usually want to loop over it, and apr_hash_first() doesn't
  * take a const hash, for various reasons.  I'm not sure that those
  * "various reasons" are actually even relevant anymore, and if
- * they're not, it might be nice to change @c apr_hash_first() so
+ * they're not, it might be nice to change apr_hash_first() so
  * read-only uses of hashes can be protected via the type system.
  *
  * Use @a pool for all allocation.  (If the caller is iterating over log
@@ -318,8 +419,7 @@ typedef svn_error_t * (*svn_commit_callback_t) (
  * processing a stream of data.
  *
  * For example, when copying data from one stream to another, do it in
- * blocks of this size; also, the standard size of one svndiff window;
- * etc.
+ * blocks of this size.
  */
 #define SVN_STREAM_CHUNK_SIZE 102400
 
@@ -367,6 +467,59 @@ svn_boolean_t svn_mime_type_is_binary (const char *mime_type);
  */
 typedef svn_error_t *(*svn_cancel_func_t) (void *cancel_baton);
 
+
+
+/**
+ * A lock object, for client & server to share.
+ *
+ * A lock represents the exclusive right to add, delete, or modify a
+ * path.  A lock is created in a repository, wholly controlled by the
+ * repository.  A "lock-token" is the lock's UUID, and can be used to
+ * learn more about a lock's fields, and or/make use of the lock.
+ * Because a lock is immutable, a client is free to not only cache the
+ * lock-token, but the lock's fields too, for convenience.
+ *
+ * Note that the 'is_dav_comment' field is wholly ignored by every
+ * library except for mod_dav_svn.  The field isn't even marshalled
+ * over the network to the client.  Assuming lock structures are
+ * created with apr_pcalloc(), a default value of 0 is universally safe.
+ *
+ * @note in the current implementation, only files are lockable.
+ *
+ * @since New in 1.2.
+ */
+typedef struct svn_lock_t
+{
+  const char *path;              /**< the path this lock applies to */
+  const char *token;             /**< unique URI representing lock */
+  const char *owner;             /**< the username which owns the lock */
+  const char *comment;           /**< (optional) description of lock  */
+  svn_boolean_t is_dav_comment;  /**< was comment made by generic DAV client? */
+  apr_time_t creation_date;      /**< when lock was made */
+  apr_time_t expiration_date;    /**< (optional) when lock will expire;
+                                      If value is 0, lock will never expire. */
+} svn_lock_t;
+
+/**
+ * Returns an @c svn_lock_t, allocated in @a pool with all fields initialized
+ * to null values.
+ *
+ * @note To allow for extending the @c svn_lock_t structure in the future
+ * releases, this function should always be used to allocate the structure.
+ *
+ * @since New in 1.2.
+ */
+svn_lock_t *
+svn_lock_create (apr_pool_t *pool);
+
+/**
+ * Return a deep copy of @a lock, allocated in @a pool.
+ *
+ * @since New in 1.2.
+ */
+svn_lock_t *
+svn_lock_dup (const svn_lock_t *lock, apr_pool_t *pool);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
