@@ -33,6 +33,7 @@
 #include "svn_path.h"
 #include "svn_io.h"
 #include "svn_config.h"
+#include "svn_props.h"
 #include "client.h"
 
 #include "svn_private_config.h"
@@ -85,12 +86,12 @@ trim_string (char **pstr)
 static svn_boolean_t
 auto_props_enumerator (const char *name,
                        const char *value,
-                       void *baton)
+                       void *baton,
+                       apr_pool_t *pool)
 {
   auto_props_baton_t *autoprops = baton;
   char *property;
   char *last_token;
-  int len;
 
   /* nothing to do here without a value */
   if (strlen (value) == 0)
@@ -106,27 +107,27 @@ auto_props_enumerator (const char *name,
   property = apr_strtok (property, ";", &last_token);
   while (property)
     {
-      char *this_value;
+      int len;
+      const char *this_value;
+      char *equal_sign = strchr (property, '=');
 
-      this_value = strchr (property, '=');
-      if (this_value)
+      if (equal_sign)
         {
-          *this_value = '\0';
-          this_value++;
-          trim_string (&this_value);
+          *equal_sign = '\0';
+          equal_sign++;
+          trim_string (&equal_sign);
+          this_value = equal_sign;
         }
       else
         {
-          this_value = (char *)"";
+          this_value = "";
         }
       trim_string (&property);
       len = strlen (property);
       if (len > 0)
         {
-          svn_string_t *propval = apr_pcalloc (autoprops->pool,
-                                               sizeof (*propval));
-          propval->data = this_value;
-          propval->len = strlen (this_value);
+          svn_string_t *propval = svn_string_create (this_value,
+                                                     autoprops->pool);
 
           apr_hash_set (autoprops->properties, property, len, propval);
           if (strcmp (property, SVN_PROP_MIME_TYPE) == 0)
@@ -168,8 +169,8 @@ svn_client__get_auto_props (apr_hash_t **properties,
 
   /* search for auto props */
   if (use_autoprops)
-    svn_config_enumerate (cfg, SVN_CONFIG_SECTION_AUTO_PROPS,
-                          auto_props_enumerator, &autoprops);
+    svn_config_enumerate2 (cfg, SVN_CONFIG_SECTION_AUTO_PROPS,
+                           auto_props_enumerator, &autoprops, pool);
 
   /* if mimetype has not been set check the file */
   if (! autoprops.mimetype)
@@ -209,9 +210,9 @@ add_file (const char *path,
   svn_boolean_t is_special;
 
   /* add the file */
-  SVN_ERR (svn_wc_add (path, adm_access, NULL, SVN_INVALID_REVNUM,
-                       ctx->cancel_func, ctx->cancel_baton,
-                       NULL, NULL, pool));
+  SVN_ERR (svn_wc_add2 (path, adm_access, NULL, SVN_INVALID_REVNUM,
+                        ctx->cancel_func, ctx->cancel_baton,
+                        NULL, NULL, pool));
 
   /* Check to see if this is a special file. */
   SVN_ERR (svn_io_check_special_path (path, &kind, &is_special, pool));
@@ -219,9 +220,10 @@ add_file (const char *path,
   if (is_special)
     {
       /* This must be a special file. */
-      SVN_ERR (svn_wc_prop_set (SVN_PROP_SPECIAL,
-                                svn_string_create (SVN_PROP_SPECIAL_VALUE, pool),
-                                path, adm_access, pool));
+      SVN_ERR (svn_wc_prop_set2
+               (SVN_PROP_SPECIAL,
+                svn_string_create (SVN_PROP_SPECIAL_VALUE, pool),
+                path, adm_access, FALSE, pool));
       mimetype = NULL;
     }
   else
@@ -239,26 +241,45 @@ add_file (const char *path,
               void *pval;
           
               apr_hash_this (hi, &pname, NULL, &pval);
-              SVN_ERR (svn_wc_prop_set (pname, pval, path, adm_access, pool));
+              /* It's probably best to pass 0 for force, so that if
+                 the autoprops say to set some weird combination,
+                 we just error and let the user sort it out. */
+              SVN_ERR (svn_wc_prop_set2 (pname, pval, path,
+                                         adm_access, FALSE, pool));
             }
         }
     }
 
   /* Report the addition to the caller. */
-  if (ctx->notify_func != NULL)
-    (*ctx->notify_func) (ctx->notify_baton, path, svn_wc_notify_add,
-                         svn_node_file,
-                         mimetype,
-                         svn_wc_notify_state_unknown,
-                         svn_wc_notify_state_unknown,
-                         SVN_INVALID_REVNUM);
+  if (ctx->notify_func2 != NULL)
+    {
+      svn_wc_notify_t *notify = svn_wc_create_notify (path, svn_wc_notify_add,
+                                                      pool);
+      notify->kind = svn_node_file;
+      notify->mime_type = mimetype;
+      (*ctx->notify_func2) (ctx->notify_baton2, notify, pool);
+    }
+
   return SVN_NO_ERROR;
 }
 
+/* Schedule directory DIRNAME recursively for addition with access baton 
+ * ADM_ACCESS.
+ *
+ * If DIRNAME (or any item below directory DIRNAME) is already scheduled for
+ * addition, add will fail and return an error unless FORCE is TRUE.
+ *
+ * Files and directories that match ignore patterns will not be added unless 
+ * NO_IGNORE is TRUE.
+ *
+ * If CTX->CANCEL_FUNC is non-null, call it with CTX->CANCEL_BATON to allow 
+ * the user to cancel the operation
+ */
 static svn_error_t *
 add_dir_recursive (const char *dirname,
                    svn_wc_adm_access_t *adm_access,
                    svn_boolean_t force,
+                   svn_boolean_t no_ignore,
                    svn_client_ctx_t *ctx,
                    apr_pool_t *pool)
 {
@@ -275,10 +296,10 @@ add_dir_recursive (const char *dirname,
     SVN_ERR (ctx->cancel_func (ctx->cancel_baton));
 
   /* Add this directory to revision control. */
-  err = svn_wc_add (dirname, adm_access,
-                    NULL, SVN_INVALID_REVNUM,
-                    ctx->cancel_func, ctx->cancel_baton,
-                    ctx->notify_func, ctx->notify_baton, pool);
+  err = svn_wc_add2 (dirname, adm_access,
+                     NULL, SVN_INVALID_REVNUM,
+                     ctx->cancel_func, ctx->cancel_baton,
+                     ctx->notify_func2, ctx->notify_baton2, pool);
   if (err && err->apr_err == SVN_ERR_ENTRY_EXISTS && force)
     svn_error_clear (err);
   else if (err)
@@ -286,7 +307,8 @@ add_dir_recursive (const char *dirname,
 
   SVN_ERR (svn_wc_adm_retrieve (&dir_access, adm_access, dirname, pool));
 
-  SVN_ERR (svn_wc_get_default_ignores (&ignores, ctx->config, pool));
+  if (!no_ignore)
+    SVN_ERR (svn_wc_get_ignores (&ignores, ctx->config, dir_access, pool));
 
   /* Create a subpool for iterative memory control. */
   subpool = svn_pool_create (pool);
@@ -306,7 +328,7 @@ add_dir_recursive (const char *dirname,
         SVN_ERR (ctx->cancel_func (ctx->cancel_baton));
       
       /* Skip over SVN admin directories. */
-      if (strcmp (this_entry.name, SVN_WC_ADM_DIR_NAME) == 0)
+      if (svn_wc_is_adm_dir (this_entry.name, subpool))
         continue;
 
       /* Skip entries for this dir and its parent.  */
@@ -315,7 +337,8 @@ add_dir_recursive (const char *dirname,
               || (this_entry.name[1] == '.' && this_entry.name[2] == '\0')))
         continue;
 
-      if (svn_cstring_match_glob_list (this_entry.name, ignores))
+      if ((!no_ignore) && svn_cstring_match_glob_list (this_entry.name,
+                                                       ignores)) 
         continue;
 
       /* Construct the full path of the entry. */
@@ -325,7 +348,7 @@ add_dir_recursive (const char *dirname,
       if (this_entry.filetype == APR_DIR)
         {
           SVN_ERR (add_dir_recursive (fullpath, dir_access, force,
-                                      ctx, subpool));
+                                      no_ignore, ctx, subpool));
         }
       else if (this_entry.filetype != APR_UNKFILE)
         {
@@ -345,7 +368,8 @@ add_dir_recursive (const char *dirname,
     {
       return svn_error_createf
         (err->apr_err, err,
-         _("Error during recursive add of '%s'"), dirname);
+         _("Error during recursive add of '%s'"),
+         svn_path_local_style (dirname, subpool));
     }
   else  /* Yes, it exited cleanly, so close the dir. */
     {
@@ -355,7 +379,8 @@ add_dir_recursive (const char *dirname,
       apr_err = apr_dir_close (dir);
       if (apr_err)
         return svn_error_wrap_apr
-          (apr_err, _("Can't close directory '%s'"), dirname);
+          (apr_err, _("Can't close directory '%s'"),
+           svn_path_local_style (dirname, subpool));
     }
 
   /* Opened by svn_wc_add */
@@ -375,6 +400,7 @@ static svn_error_t *
 add (const char *path, 
      svn_boolean_t recursive,
      svn_boolean_t force,
+     svn_boolean_t no_ignore,
      svn_wc_adm_access_t *adm_access,
      svn_client_ctx_t *ctx,
      apr_pool_t *pool)
@@ -384,13 +410,13 @@ add (const char *path,
 
   SVN_ERR (svn_io_check_path (path, &kind, pool));
   if ((kind == svn_node_dir) && recursive)
-    err = add_dir_recursive (path, adm_access, force, ctx, pool);
+    err = add_dir_recursive (path, adm_access, force, no_ignore, ctx, pool);
   else if (kind == svn_node_file)
     err = add_file (path, ctx, adm_access, pool);
   else
-    err = svn_wc_add (path, adm_access, NULL, SVN_INVALID_REVNUM,
-                      ctx->cancel_func, ctx->cancel_baton,
-                      ctx->notify_func, ctx->notify_baton, pool);
+    err = svn_wc_add2 (path, adm_access, NULL, SVN_INVALID_REVNUM,
+                       ctx->cancel_func, ctx->cancel_baton,
+                       ctx->notify_func2, ctx->notify_baton2, pool);
 
   /* Ignore SVN_ERR_ENTRY_EXISTS when FORCE is set.  */
   if (err && err->apr_err == SVN_ERR_ENTRY_EXISTS && force)
@@ -404,19 +430,22 @@ add (const char *path,
 
 
 svn_error_t *
-svn_client_add (const char *path, 
-                svn_boolean_t recursive,
-                svn_client_ctx_t *ctx,
-                apr_pool_t *pool)
+svn_client_add3 (const char *path, 
+                 svn_boolean_t recursive,
+                 svn_boolean_t force,
+                 svn_boolean_t no_ignore,
+                 svn_client_ctx_t *ctx,
+                 apr_pool_t *pool)
 {
   svn_error_t *err, *err2;
   svn_wc_adm_access_t *adm_access;
   const char *parent_path = svn_path_dirname (path, pool);
 
-  SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, parent_path,
-                             TRUE, 0, pool));
+  SVN_ERR (svn_wc_adm_open3 (&adm_access, NULL, parent_path,
+                             TRUE, 0, ctx->cancel_func, ctx->cancel_baton,
+                             pool));
 
-  err = add (path, recursive, FALSE, adm_access, ctx, pool);
+  err = add (path, recursive, force, no_ignore, adm_access, ctx, pool);
   
   err2 = svn_wc_adm_close (adm_access);
   if (err2)
@@ -432,31 +461,22 @@ svn_client_add (const char *path,
 
 
 svn_error_t *
-svn_client_add2 (const char *path, 
+svn_client_add2 (const char *path,
                  svn_boolean_t recursive,
                  svn_boolean_t force,
                  svn_client_ctx_t *ctx,
                  apr_pool_t *pool)
 {
-  svn_error_t *err, *err2;
-  svn_wc_adm_access_t *adm_access;
-  const char *parent_path = svn_path_dirname (path, pool);
+  return svn_client_add3 (path, recursive, force, FALSE, ctx, pool);
+}
 
-  SVN_ERR (svn_wc_adm_open2 (&adm_access, NULL, parent_path,
-                             TRUE, 0, pool));
-
-  err = add (path, recursive, force, adm_access, ctx, pool);
-  
-  err2 = svn_wc_adm_close (adm_access);
-  if (err2)
-    {
-      if (err)
-        svn_error_clear (err2);
-      else
-        err = err2;
-    }
-
-  return err;
+svn_error_t *
+svn_client_add (const char *path, 
+                svn_boolean_t recursive,
+                svn_client_ctx_t *ctx,
+                apr_pool_t *pool)
+{
+  return svn_client_add3 (path, recursive, FALSE, FALSE, ctx, pool);
 }
 
 
@@ -468,19 +488,19 @@ path_driver_cb_func (void **dir_baton,
                      apr_pool_t *pool)
 {
   const svn_delta_editor_t *editor = callback_baton;
+  SVN_ERR (svn_path_check_valid (path, pool));
   return editor->add_directory (path, parent_baton, NULL,
                                 SVN_INVALID_REVNUM, pool, dir_baton);
 }
 
 
 static svn_error_t *
-mkdir_urls (svn_client_commit_info_t **commit_info,
+mkdir_urls (svn_commit_info_t **commit_info_p,
             const apr_array_header_t *paths,
             svn_client_ctx_t *ctx,
             apr_pool_t *pool)
 {
-  void *ra_baton, *session;
-  svn_ra_plugin_t *ra_lib;
+  svn_ra_session_t *ra_session;
   const svn_delta_editor_t *editor;
   void *edit_baton;
   void *commit_baton;
@@ -528,9 +548,9 @@ mkdir_urls (svn_client_commit_info_t **commit_info,
     }
 
   /* Create new commit items and add them to the array. */
-  if (ctx->log_msg_func)
+  if (ctx->log_msg_func || ctx->log_msg_func2)
     {
-      svn_client_commit_item_t *item;
+      svn_client_commit_item2_t *item;
       const char *tmp_file;
       apr_array_header_t *commit_items 
         = apr_array_make (pool, targets->nelts, sizeof (item));
@@ -541,25 +561,23 @@ mkdir_urls (svn_client_commit_info_t **commit_info,
           item = apr_pcalloc (pool, sizeof (*item));
           item->url = svn_path_join (common, path, pool);
           item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
-          APR_ARRAY_PUSH (commit_items, svn_client_commit_item_t *) = item;
+          APR_ARRAY_PUSH (commit_items, svn_client_commit_item2_t *) = item;
         }
-      SVN_ERR ((*ctx->log_msg_func) (&log_msg, &tmp_file, commit_items, 
-                                     ctx->log_msg_baton, pool));
+
+      SVN_ERR (svn_client__get_log_msg(&log_msg, &tmp_file, commit_items,
+                                       ctx, pool));
+
       if (! log_msg)
         return SVN_NO_ERROR;
     }
   else
     log_msg = "";
 
-  /* Get the RA vtable that matches URL. */
-  SVN_ERR (svn_ra_init_ra_libs (&ra_baton, pool));
-  SVN_ERR (svn_ra_get_ra_library (&ra_lib, ra_baton, common, pool));
-
   /* Open an RA session for the URL. Note that we don't have a local
      directory, nor a place to put temp files. */
-  SVN_ERR (svn_client__open_ra_session (&session, ra_lib, common, NULL,
-                                        NULL, NULL, FALSE, TRUE,
-                                        ctx, pool));
+  SVN_ERR (svn_client__open_ra_session_internal (&ra_session, common, NULL,
+                                                 NULL, NULL, FALSE, TRUE,
+                                                 ctx, pool));
 
   /* URI-decode each target. */
   for (i = 0; i < targets->nelts; i++)
@@ -570,11 +588,13 @@ mkdir_urls (svn_client_commit_info_t **commit_info,
     }
 
   /* Fetch RA commit editor */
-  SVN_ERR (svn_client__commit_get_baton (&commit_baton, commit_info, pool));
-  SVN_ERR (ra_lib->get_commit_editor (session, &editor, &edit_baton,
-                                      log_msg, svn_client__commit_callback,
-                                      commit_baton, pool));
-
+  SVN_ERR (svn_client__commit_get_baton (&commit_baton, commit_info_p, pool));
+  SVN_ERR (svn_ra_get_commit_editor (ra_session, &editor, &edit_baton,
+                                     log_msg, svn_client__commit_callback,
+                                     commit_baton, 
+                                       NULL, TRUE, /* No lock tokens */
+                                     pool));
+  
   /* Call the path-based editor driver. */
   err = svn_delta_path_driver (editor, edit_baton, SVN_INVALID_REVNUM, 
                                targets, path_driver_cb_func, 
@@ -594,17 +614,17 @@ mkdir_urls (svn_client_commit_info_t **commit_info,
 
 
 svn_error_t *
-svn_client_mkdir (svn_client_commit_info_t **commit_info,
-                  const apr_array_header_t *paths,
-                  svn_client_ctx_t *ctx,
-                  apr_pool_t *pool)
+svn_client_mkdir2 (svn_commit_info_t **commit_info_p,
+                   const apr_array_header_t *paths,
+                   svn_client_ctx_t *ctx,
+                   apr_pool_t *pool)
 {
   if (! paths->nelts)
     return SVN_NO_ERROR;
   
   if (svn_path_is_url (APR_ARRAY_IDX (paths, 0, const char *)))
     {
-      SVN_ERR (mkdir_urls (commit_info, paths, ctx, pool));
+      SVN_ERR (mkdir_urls (commit_info_p, paths, ctx, pool));
     }
   else
     {
@@ -617,28 +637,44 @@ svn_client_mkdir (svn_client_commit_info_t **commit_info,
         {
           const char *path = APR_ARRAY_IDX (paths, i, const char *);
 
+          svn_pool_clear (subpool);
+
           /* See if the user wants us to stop. */
           if (ctx->cancel_func)
             SVN_ERR (ctx->cancel_func (ctx->cancel_baton));
 
-          SVN_ERR (svn_io_dir_make (path, APR_OS_DEFAULT, pool));
-          err = svn_client_add (path, FALSE, ctx, pool);
+          SVN_ERR (svn_io_dir_make (path, APR_OS_DEFAULT, subpool));
+          err = svn_client_add3 (path, FALSE, FALSE, FALSE, ctx, subpool);
 
-          /* Trying to add a directory with the same name as a file that is
-             scheduled for deletion is not supported.  Leaving an unversioned
-             directory makes the working copy hard to use.  */
-          if (err && err->apr_err == SVN_ERR_WC_NODE_KIND_CHANGE)
+          /* We just created a new directory, but couldn't add it to
+             version control. Don't leave unversioned directoies behind. */
+          if (err)
             {
-              svn_error_t *err2 = svn_io_remove_dir (path, pool);
-              if (err2)
-                svn_error_clear (err2);
+              /* ### If this returns an error, should we link it onto
+                 err instead, so that the user is warned that we just
+                 created an unversioned directory? */
+              svn_error_clear (svn_io_remove_dir (path, subpool));
+              return err;
             }
-          SVN_ERR (err);
-
-          svn_pool_clear (subpool);
         }
       svn_pool_destroy (subpool);
     }
 
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_client_mkdir (svn_client_commit_info_t **commit_info_p,
+                  const apr_array_header_t *paths,
+                  svn_client_ctx_t *ctx,
+                  apr_pool_t *pool)
+{
+  svn_commit_info_t *commit_info = NULL;
+  svn_error_t *err;
+
+  err = svn_client_mkdir2 (&commit_info, paths, ctx, pool);
+  /* These structs have the same layout for the common fields. */
+  *commit_info_p = (svn_client_commit_info_t *) commit_info;
+  return err;
+}
