@@ -358,8 +358,17 @@ svn_ra_dav__get_file_revs (svn_ra_session_t *session,
   if (http_status == 501)
     return svn_error_create (SVN_ERR_RA_NOT_IMPLEMENTED, err,
                              _("'get-file-revs' REPORT not implemented"));
+
+  /* rb.err contains the relevant error if the response was aborted by
+   * a callback returning NE_XML_ABORT; always return that error if
+   * present. */
+  if (rb.err != NULL)
+    {
+      if (err)
+        svn_error_clear (err);
+      return rb.err;
+    }
   SVN_ERR (err);
-  SVN_ERR (rb.err);
 
   /* Caller expects at least one revision.  Signal error otherwise. */
   if (!SVN_IS_VALID_REVNUM(rb.revnum))
