@@ -20,7 +20,8 @@
 #include <string.h>
 #include <svn_path.h>
 #include <apr_general.h>
-#include "svn_test.h"
+
+#include "../svn_test.h"
 
 /* Using a symbol, because I tried experimenting with different
    representations */
@@ -29,6 +30,7 @@
 static svn_error_t *
 test_path_is_child (const char **msg,
                     svn_boolean_t msg_only,
+                    svn_test_opts_t *opts,
                     apr_pool_t *pool)
 {
   int i, j;
@@ -91,6 +93,7 @@ test_path_is_child (const char **msg,
 static svn_error_t *
 test_path_split (const char **msg,
                  svn_boolean_t msg_only,
+                 svn_test_opts_t *opts,
                  apr_pool_t *pool)
 {
   apr_size_t i;
@@ -143,9 +146,10 @@ test_path_split (const char **msg,
 static svn_error_t *
 test_is_url (const char **msg,
              svn_boolean_t msg_only,
+             svn_test_opts_t *opts,
              apr_pool_t *pool)
 {
-  int i;
+  apr_size_t i;
 
   /* Paths to test. */
   static const char * const paths[] = { 
@@ -154,6 +158,8 @@ test_is_url (const char **msg,
     "http://svn.collab.net/repos/svn",
     "scheme/with://slash/",
     "file:///path/to/repository",
+    "file://",
+    "file:/",
   };
 
   /* Expected results of the tests. */
@@ -162,14 +168,17 @@ test_is_url (const char **msg,
     FALSE,
     TRUE,
     FALSE,
-    TRUE };
+    TRUE,
+    TRUE,
+    FALSE,
+  };
 
   *msg = "test svn_path_is_url";
 
   if (msg_only)
     return SVN_NO_ERROR;
 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < sizeof (paths) / sizeof (paths[0]); i++)
     {
       svn_boolean_t retval;
 
@@ -188,6 +197,7 @@ test_is_url (const char **msg,
 static svn_error_t *
 test_is_uri_safe (const char **msg,
                   svn_boolean_t msg_only,
+                  svn_test_opts_t *opts,
                   apr_pool_t *pool)
 {
   apr_size_t i;
@@ -241,6 +251,7 @@ test_is_uri_safe (const char **msg,
 static svn_error_t *
 test_uri_encode (const char **msg,
                  svn_boolean_t msg_only,
+                 svn_test_opts_t *opts,
                  apr_pool_t *pool)
 {
   int i;
@@ -294,6 +305,7 @@ test_uri_encode (const char **msg,
 static svn_error_t *
 test_uri_decode (const char **msg,
                  svn_boolean_t msg_only,
+                 svn_test_opts_t *opts,
                  apr_pool_t *pool)
 {
   int i;
@@ -333,6 +345,7 @@ test_uri_decode (const char **msg,
 static svn_error_t *
 test_uri_autoescape (const char **msg,
                      svn_boolean_t msg_only,
+                     svn_test_opts_t *opts,
                      apr_pool_t *pool)
 {
   static const char *paths[3][2] = {
@@ -369,6 +382,7 @@ test_uri_autoescape (const char **msg,
 static svn_error_t *
 test_uri_from_iri (const char **msg,
                    svn_boolean_t msg_only,
+                   svn_test_opts_t *opts,
                    apr_pool_t *pool)
 {
   /* We have to code the IRIs like this because the compiler might translate
@@ -376,12 +390,12 @@ test_uri_from_iri (const char **msg,
      but here we are hard-coding UTF-8.  But we all read UTF-8 codes like
      poetry, don't we. */
   static const char p1[] = {
-    0x66, 0x69, 0x6C, 0x65, 0x3A, 0x2F, 0x2F, 0x2F,
-    0x72, 0xC3, 0xA4, 0x6B, 0x73, 0x6D, 0xC3, 0xB6, 0x72,
-    0x67, 0xC3, 0xA5, 0x73, 0 };
+    '\x66', '\x69', '\x6C', '\x65', '\x3A', '\x2F', '\x2F', '\x2F',
+    '\x72', '\xC3', '\xA4', '\x6B', '\x73', '\x6D', '\xC3', '\xB6', '\x72',
+    '\x67', '\xC3', '\xA5', '\x73', '\0' };
   static const char p2[] = {
-    0x66, 0x69, 0x6C, 0x65, 0x3A, 0x2F, 0x2F, 0x2F,
-    0x61, 0x62, 0x25, 0x32, 0x30, 0x63, 0x64, 0 };
+    '\x66', '\x69', '\x6C', '\x65', '\x3A', '\x2F', '\x2F', '\x2F',
+    '\x61', '\x62', '\x25', '\x32', '\x30', '\x63', '\x64', '\0' };
   static const char *paths[2][2] = {
     { p1,
       "file:///r%C3%A4ksm%C3%B6rg%C3%A5s" },
@@ -390,7 +404,7 @@ test_uri_from_iri (const char **msg,
   };
   int i;
 
-  *msg = "testing svn_uri_from_iri";
+  *msg = "test svn_path_uri_from_iri";
 
   if (msg_only)
     return SVN_NO_ERROR;
@@ -401,7 +415,7 @@ test_uri_from_iri (const char **msg,
       if (strcmp (paths[i][1], uri) != 0)
         return svn_error_createf
           (SVN_ERR_TEST_FAILED, NULL,
-           "svn_uri_from_iri on '%s' returned '%s' instead of '%s'",
+           "svn_path_uri_from_iri on '%s' returned '%s' instead of '%s'",
            paths[i][0], uri, paths[i][1]);
       if (strcmp (paths[i][0], uri) == 0
           && paths[i][0] != uri)
@@ -417,6 +431,7 @@ test_uri_from_iri (const char **msg,
 static svn_error_t *
 test_join (const char **msg,
            svn_boolean_t msg_only,
+           svn_test_opts_t *opts,
            apr_pool_t *pool)
 {
   int i;
@@ -515,6 +530,7 @@ test_join (const char **msg,
 static svn_error_t *
 test_basename (const char **msg,
                svn_boolean_t msg_only,
+               svn_test_opts_t *opts,
                apr_pool_t *pool)
 {
   int i;
@@ -560,6 +576,7 @@ test_basename (const char **msg,
 static svn_error_t *
 test_decompose (const char **msg,
                 svn_boolean_t msg_only,
+                svn_test_opts_t *opts,
                 apr_pool_t *pool)
 {
   static const char * const paths[] = {
@@ -616,6 +633,7 @@ test_decompose (const char **msg,
 static svn_error_t *
 test_canonicalize (const char **msg,
                    svn_boolean_t msg_only,
+                   svn_test_opts_t *opts,
                    apr_pool_t *pool)
 {
   const char *paths[][2] = {
@@ -652,19 +670,19 @@ test_canonicalize (const char **msg,
     { "http://hst",           "http://hst" },
     { "http://hst/foo/../bar","http://hst/foo/../bar" },
     { "http://hst/",          "http://hst" },
-#ifdef WIN32 
+#if defined(WIN32) || defined(__CYGWIN__)
     /* We permit UNC paths on Windows.  By definition UNC
      * paths must have two components so we should remove the
      * double slash if there is only one component. */
     { "//hst/foo",            "//hst/foo" },
     { "//hst",                "/hst" },
     { "//hst/./",             "/hst" },
-#endif /* WIN32 */
+#endif /* WIN32 or Cygwin */
     { NULL, NULL }
   };
   int i;
 
-  *msg = "test svn_path_decompose";
+  *msg = "test svn_path_canonicalize";
   if (msg_only)
     return SVN_NO_ERROR;
 
@@ -687,6 +705,7 @@ test_canonicalize (const char **msg,
 static svn_error_t *
 test_remove_component (const char **msg,
                        svn_boolean_t msg_only,
+                       svn_test_opts_t *opts,
                        apr_pool_t *pool)
 {
   const char *paths[][2] = {
