@@ -16,8 +16,8 @@
  * ====================================================================
  */
 
+#include "../svn_test.h"
 #include "../../libsvn_subr/utf_impl.h"
-#include "svn_test.h"
 
 /* Random number seed.  Yes, it's global, just pretend you can't see it. */
 static apr_uint32_t diff_diff3_seed;
