@@ -38,6 +38,7 @@
 #include "svn_client.h"
 #include "svn_pools.h"
 #include "svn_config.h"
+#include "svn_fs.h"
 
 
 /* Display a prompt and read a one-line response into the provided buffer,
@@ -146,14 +147,22 @@ main (int argc, const char **argv)
      understand how to properly use/free subpools. */
   pool = svn_pool_create (NULL);
 
-  /* Make sure the ~/.subversion run-time config files exist */  
-  err = svn_config_ensure (NULL, pool);
+  /* Initialize the FS library. */
+  err = svn_fs_initialize (pool);
   if (err)
     {
       /* For functions deeper in the stack, we usually use the
          SVN_ERR() exception-throwing macro (see svn_error.h).  At the
-         top level, we catch & print the error with svn_handle_error(). */
-      svn_handle_error (err, stderr, 0);
+         top level, we catch & print the error with svn_handle_error2(). */
+      svn_handle_error2 (err, stderr, FALSE, "minimal_client: ");
+      return EXIT_FAILURE;
+    }
+
+  /* Make sure the ~/.subversion run-time config files exist */  
+  err = svn_config_ensure (NULL, pool);
+  if (err)
+    {
+      svn_handle_error2 (err, stderr, FALSE, "minimal_client: ");
       return EXIT_FAILURE;
     }
 
@@ -162,17 +171,30 @@ main (int argc, const char **argv)
     /* Initialize and allocate the client_ctx object. */
     if ((err = svn_client_create_context (&ctx, pool)))
       {
-        svn_handle_error (err, stderr, 0);
+        svn_handle_error2 (err, stderr, FALSE, "minimal_client: ");
         return EXIT_FAILURE;
       }
     
     /* Load the run-time config file into a hash */
     if ((err = svn_config_get_config (&(ctx->config), NULL, pool)))
       {
-        svn_handle_error (err, stderr, 0);
+        svn_handle_error2 (err, stderr, FALSE, "minimal_client: ");
         return EXIT_FAILURE;
       }
 
+#ifdef WIN32
+    /* Set the working copy administrative directory name. */
+    if (getenv ("SVN_ASP_DOT_NET_HACK"))
+      {
+        err = svn_wc_set_adm_dir ("_svn", pool);
+        if (err)
+          {
+            svn_handle_error2 (err, stderr, FALSE, "minimal_client: ");
+            return EXIT_FAILURE;
+          }
+      }
+#endif
+
     /* Depending on what your client does, you'll want to read about
        (and implement) the various callback function types below.  */
 
@@ -228,7 +250,7 @@ main (int argc, const char **argv)
                        ctx, pool);
   if (err)
     {
-      svn_handle_error (err, stderr, 0);
+      svn_handle_error2 (err, stderr, FALSE, "minimal_client: ");
       return EXIT_FAILURE;
     }
 
